﻿------------------------------
--      Are you local?      --
------------------------------

local name = "CThun Assist"
local L = AceLibrary("AceLocale-2.2"):new("BigWigs"..name)
local boss = BB["Eye of C'Thun"]
local cthun = BB["C'Thun"]
local tablet = AceLibrary("Tablet-2.0")
local paintchips = AceLibrary("PaintChips-2.0")
local roster = nil
local stage2 = nil
local lastplayed = 0
local playername = nil
local combat = nil

----------------------------
--      Localization      --
----------------------------

L:RegisterTranslations("enUS", function() return {
	cmd = "CthunAssist",
	["Big Wigs CThun Assist"] = true,

	sound = "Sound",
	sound_desc = "Play sound on proximity.",

	["Phase 1 Too Close"] = true,
	["Phase 2 Stomach"] = true,
	["Nobody"] = true,
} end )

--Chinese Translate by 月色狼影@CWDG
--CWDG site: http://Cwowaddon.com
L:RegisterTranslations("zhCN", function() return {
	["Big Wigs CThun Assist"] = "Big Wigs克苏恩助手",
	
	sound = "声音",
	sound_desc = "接近时声音警报.",

	["Phase 1 Too Close"] = "距离太近！",
	["Phase 2 Stomach"] = "吞噬出现",
	["Nobody"] = "没有人",
} end )

L:RegisterTranslations("koKR", function() return {
	sound = "경고음",
	sound_desc = "근접했을 때 경고음.",

	["Phase 1 Too Close"] = "1단계 너무 가까움",
	["Phase 2 Stomach"] = "2단계 뱃속",
	["Nobody"] = "아무도 없음",
} end )

----------------------------------
--      Module Declaration      --
----------------------------------

local mod = BigWigs:NewModule(name)
mod.synctoken = name
mod.zonename = BZ["Ahn'Qiraj"]
mod.enabletrigger = boss
mod.toggleoptions = {"sound"}
mod.revision = tonumber(("$Revision: 61747 $"):sub(12, -3))
mod.external = true

------------------------------
--      Initialization      --
------------------------------

function mod:OnEnable()
	if not roster then roster = AceLibrary("Roster-2.1") end

	stage2 = nil
	lastplayed = 0
	playername = UnitName("player")

	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
	self:RegisterEvent("BigWigs_RecvSync")

	self:ShowTablet()
end

function mod:OnDisable()
	roster = nil
	stage2 = nil
	self:HideTablet()
end

------------------------------
--      Events              --
------------------------------

function mod:CHAT_MSG_COMBAT_HOSTILE_DEATH(msg)
	if msg == UNITDIESOTHER:format(cthun) then
		BigWigs:ToggleModuleActive(self, false)
	end
end

function mod:BigWigs_RecvSync(sync)
	if sync == "CThunStart" then
		stage2 = nil
	elseif sync == "CThunP2Start" then
		stage2 = true
	end
end

------------------------------
--      Tablet              --
------------------------------

function mod:OnTooltipUpdate()
	if not tablet:IsRegistered("BigWigs_CThunAssist") then return end

	local headertext = stage2 and L["Phase 2 Stomach"] or L["Phase 1 Too Close"]
	local cat = tablet:AddCategory(
		"columns", 1,
		"text", "|cffcccccc"..headertext.."|r",
		"justify", "CENTER",
		"child_justify", "CENTER"
	)

	local tooclose = 0
	local added = false

	for n, u in pairs(roster.roster) do
		if u and u.name and u.class ~= "PET" and not UnitIsDeadOrGhost(u.unitid) then
			if stage2 then
				for a=1,16 do
					local t,c = UnitDebuff(u.unitid,a)
					if not t then break end
					if t == "Interface\\Icons\\Ability_Creature_Disease_02" then
						cat:AddLine("text", "|cff"..paintchips:GetHex(u.class)..u.name.."|r")
						added = true
					end
				end
			else
				if tooclose < 5 and u.name ~= playername and CheckInteractDistance(u.unitid, 3) then
					cat:AddLine("text", "|cff"..paintchips:GetHex(u.class)..u.name.."|r")
					tooclose = tooclose + 1
				end
			end
		end
	end

	if not added and tooclose == 0 then
		cat:AddLine("text", "|cff777777"..L["Nobody"].."|r")
	end

	local t = time()
	if tooclose > 0 and t > lastplayed + 1 then
		lastplayed = time()
		if self.db.profile.sound and UnitAffectingCombat("player") then self:TriggerEvent("BigWigs_Sound", "Alarm") end
	end
end

function mod:ShowTablet()
	if not tablet:IsRegistered("BigWigs_CThunAssist") then
		tablet:Register("BigWigs_CThunAssist",
			"children",
				function()
					tablet:SetTitle(L["Big Wigs CThun Assist"])
					self:OnTooltipUpdate()
				end,
			"clickable", true,
			"showTitleWhenDetached", true,
			"showHintWhenDetached", true,
			"cantAttach", true
		)
	end

	if not self:IsEventScheduled("bwcthunassistupdate") then
		self:ScheduleRepeatingEvent("bwcthunassistupdate", function() tablet:Refresh("BigWigs_CThunAssist") end, .1)
	end

	if tablet:IsAttached("BigWigs_CThunAssist") then
		tablet:Detach("BigWigs_CThunAssist")
	end
end

function mod:HideTablet()
	if not tablet:IsRegistered("BigWigs_CThunAssist") then return end

	self:CancelScheduledEvent("bwcthunassistupdate")

	tablet:Attach("BigWigs_CThunAssist")
end
