﻿--[[

Addon to display raid members in the portal for the Kalecgos encounter in Sunwell Plateau
Thanks to the BigWigs proximity module by Ammo from which a lot of this code is copied

]]

------------------------------
--      Are you local?      --
------------------------------

local name = "Kalecgos Portals"
local L = AceLibrary("AceLocale-2.2"):new("BigWigs"..name)
local boss = BB["Kalecgos"]
local LBW = AceLibrary("AceLocale-2.2"):new("BigWigs"..boss)

local BWKalecFrame = nil

local insideIt = {} -- List of players who are in the portal.

local table_insert = table.insert
local table_concat = table.concat
local fmt = string.format
local GetNumRaidMembers = GetNumRaidMembers
local GetRaidRosterInfo = GetRaidRosterInfo
local UnitDebuff = UnitDebuff
local UnitClass = UnitClass
local pairs = pairs

local hexColors = {}
for k, v in pairs(RAID_CLASS_COLORS) do
	hexColors[k] = ("|cff%02x%02x%02x"):format(v.r * 255, v.g * 255, v.b * 255)
end

-- Helper table to cache colored player names.
local coloredNames = setmetatable({}, {__index =
	function(self, key)
		if type(key) == "nil" then return nil end
		local _, class = UnitClass(key)
		if class then
			self[key] = hexColors[class] .. key .. "|r"
			return self[key]
		else
			return key
		end
	end
})

----------------------------
--      Localization      --
----------------------------

L:RegisterTranslations("enUS", function() return {
	cmd = "KalecgosPortals",

	on = "On",
	on_desc = "Toggle BigWigs_KalecgosPortals on or off.",

	["In The Portal"] = true,
	["%s (%d)"] = true,
	["|cff777777Nobody|r"] = true,
	font = "Fonts\\FRIZQT__.TTF",
} end )

L:RegisterTranslations("koKR", function() return {
	on = "사용",
	on_desc = "BigWigs_KalecgosPortals 켜거나 끕니다.",

	["In The Portal"] = "내부 인원",
	["%s (%d)"] = "%s (%d)",
	["|cff777777Nobody|r"] = "|cff777777아무도 없음|r",
	font = "Fonts\\2002.TTF",
} end )

L:RegisterTranslations("frFR", function() return {
	on = "Actif",
	on_desc = "Active ou non BigWigs_KalecgosPortals.",

	["In The Portal"] = "Dans le portail",
	["%s (%d)"] = "%s (%d)",
	["|cff777777Nobody|r"] = "|cff777777Personne|r",
} end )

L:RegisterTranslations("zhCN", function() return {
	on = "启用",
	on_desc = "启用或禁用 BigWigs 卡雷苟斯传送门提示插件。",

	["In The Portal"] = "位于传送门",
	["%s (%d)"] = "%s (%d)",
	["|cff777777Nobody|r"] = "|cff777777没有玩家|r",
	font = "Fonts\\ZYKai_T.TTF",
} end )

L:RegisterTranslations("zhTW", function() return {
	on = "啟用",
	on_desc = "啟用或關閉卡雷苟斯傳送門提示模組",

	["In The Portal"] = "在傳送門內",
	["%s (%d)"] = "%s (%d)",
	["|cff777777Nobody|r"] = "|cff777777沒有玩家|r",
	font = "Fonts\\bHEI01B.TTF",
} end )

L:RegisterTranslations("deDE", function() return {
	on = "An",
	on_desc = "Schalte BigWigs_KalecgosPortals an oder aus.",

	["In The Portal"] = "In dem Portal",
	["%s (%d)"] = "%s (%d)",
	["|cff777777Nobody|r"] = "|cff777777Niemand|r",
	font = "Fonts\\FRIZQT__.TTF",
} end )

L:RegisterTranslations("ruRU", function() return {
	on = "On",
	on_desc = "Toggle BigWigs_KalecgosPortals on or off.",

	["In The Portal"] = "In The Portal",
	["%s (%d)"] = "%s (%d)",
	["|cff777777Nobody|r"] = "|cff777777Nobody|r",
	font = "Fonts\\FRIZQT__.TTF",
} end )

----------------------------------
--      Module Declaration      --
----------------------------------

local mod = BigWigs:NewModule(name)
mod.synctoken = name
mod.zonename = BZ["Sunwell Plateau"]
mod.enabletrigger = boss
mod.toggleoptions = {"on"}
mod.revision = tonumber(("$Revision: 81683 $"):sub(12, -3))
mod.external = true

------------------------------
--      Initialization      --
------------------------------

function mod:OnEnable()
	self:RegisterEvent("BigWigs_RecvSync")
	self:RegisterEvent("CHAT_MSG_MONSTER_YELL")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")

	if BWKalecFrame then
		BWKalecFrame:Hide()
	end
end

function mod:OnDisable()
	if BWKalecFrame then
		BWKalecFrame:Hide()
	end
end

------------------------------
--      Events              --
------------------------------

function mod:CHAT_MSG_MONSTER_YELL(msg)
	if not self.db.profile.on then return end

	if msg == LBW["engage_trigger"] then
		if not BWKalecFrame then
			self:SetupFrames()
		else
			BWKalecFrame:Show()
		end

		self:ScheduleRepeatingEvent("BWKalecFrameCheck", self.Update, 2, self)
	end
end

function mod:PLAYER_REGEN_ENABLED()
	if not self.db.profile.on then return end

	local go = self:Scan()
	local running = self:IsEventScheduled("Kalecgos_CheckWipe")
	if not go then
		self:TriggerEvent("BigWigs_RebootModule", self)
	elseif not running then
		self:ScheduleRepeatingEvent("Kalecgos_CheckWipe", self.PLAYER_REGEN_ENABLED, 2, self)
	end
end

function mod:BigWigs_RecvSync(sync, rest)
	if sync == "BWBossDeath" and rest == "Kalecgos" then
		BigWigs:ToggleModuleActive(self, false)
		if BWKalecFrame then
			BWKalecFrame:Hide()
		end
	end
end

local realmID = GetSpellInfo(46021) --Spectral Realm
function mod:Update()
	local groups = {};
	for i = 1,8 do
		groups[i] = 0;
	end
	for i = 1, GetNumRaidMembers() do
		local iX = 1
		local name, _, subGroup = GetRaidRosterInfo(i)
		while UnitDebuff(name, iX) do
			local buff = UnitDebuff(name, iX)
			if buff == realmID then
				local text = fmt(L["%s (%d)"], coloredNames[name], subGroup)
				groups[subGroup] = groups[subGroup] + 1 ;
				table_insert(insideIt, text)
				break
			end
			iX = iX + 1
		end
	end
	if #insideIt == 0 then
		BWKalecFrame.text:SetText(L["|cff777777Nobody|r"])
	else
		BWKalecFrame.text:SetText(table_concat(insideIt, "\n"))
		for k in pairs(insideIt) do insideIt[k] = nil end
		for i = 1,5 do
			--if (groups[i] >0) then
				--DEFAULT_CHAT_FRAME:AddMessage("group "..i.." : "..groups[i]);				
			--end
			BWKalecFrame.group[i]:SetText(groups[i]);
		end		
	end
end

function mod:SetupFrames()
	BWKalecFrame = CreateFrame("Frame", "BWKalecFrameDisplay", UIParent)

	BWKalecFrame:SetWidth(170)
	BWKalecFrame:SetHeight(270)

	BWKalecFrame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\AddOns\\BigWigs\\Textures\\otravi-semi-full-border", edgeSize = 32,
		insets = {left = 1, right = 1, top = 20, bottom = 1},
	})

	BWKalecFrame:SetBackdropColor(0.733,0.749,1)
	BWKalecFrame:ClearAllPoints()
	BWKalecFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	BWKalecFrame:EnableMouse(true)
	BWKalecFrame:RegisterForDrag("LeftButton")
	BWKalecFrame:SetClampedToScreen(true)
	BWKalecFrame:SetMovable(true)
	BWKalecFrame:SetScript("OnDragStart", function() this:StartMoving() end)
	BWKalecFrame:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		self:SavePosition()
	end)

	local cheader = BWKalecFrame:CreateFontString(nil, "OVERLAY")
	cheader:ClearAllPoints()
	cheader:SetWidth(190)
	cheader:SetHeight(15)
	cheader:SetPoint("TOP", BWKalecFrame, "TOP", 0, -14)
	cheader:SetFont(L["font"], 12)
	cheader:SetJustifyH("CENTER")
	--cheader:SetText(L["In The Portal"])
	cheader:SetText("")
	cheader:SetShadowOffset(.8, -.8)
	cheader:SetShadowColor(0, 0, 0, 1)
	BWKalecFrame.cheader = cheader
	
	BWKalecFrame.group = {};
	
	for i = 1, 5 do 
		local group = BWKalecFrame:CreateFontString(nil, "OVERLAY")
		group:ClearAllPoints()
		group:SetWidth(40)
		group:SetHeight(15)
		group:SetPoint("TOP", BWKalecFrame, "TOP", -105+i*32, -14)
		group:SetFont(L["font"], 12)
		group:SetJustifyH("CENTER")
		group:SetText("0")
		group:SetShadowOffset(.8, -.8)
		group:SetShadowColor(0, 0, 0, 1)
		BWKalecFrame.group[i] = group	
	end

	local text = BWKalecFrame:CreateFontString(nil, "OVERLAY")
	text:ClearAllPoints()
	text:SetWidth( 190 )
	text:SetHeight( 250 )
	text:SetPoint( "TOP", BWKalecFrame, "TOP", 0, -35 )
	text:SetJustifyH("CENTER")
	text:SetJustifyV("TOP")
	text:SetFont(L["font"], 12)
	BWKalecFrame.text = text

	local x = self.db.profile.posx
	local y = self.db.profile.posy
	if x and y then
		local s = BWKalecFrame:GetEffectiveScale()
		BWKalecFrame:ClearAllPoints()
		BWKalecFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
	end

	for k in pairs(insideIt) do insideIt[k] = nil end
end

function mod:SavePosition()
	if not BWKalecFrame then self:SetupFrames() end

	local s = BWKalecFrame:GetEffectiveScale()
	self.db.profile.posx = BWKalecFrame:GetLeft() * s
	self.db.profile.posy = BWKalecFrame:GetTop() * s
end
