--[[
--
-- BigWigs Strategy Module 
--
-- Warns the raid when Void Blast is reflected.
--
--]]

------------------------------
--      Are you local?      --
------------------------------

local name = "Void Blast Reflect"
local L = AceLibrary("AceLocale-2.2"):new("BigWigs"..name)

local times = nil

L:RegisterTranslations("enUS", function() return {
	["Void Blast Reflect"] = true,
	["voidblastreflect"] = true,
	["Optionally broadcast the raid when void blast is reflected."] = true,
	["broadcast"] = true,
	["Broadcast"] = true,
	["Toggle broadcasting."] = true,
	["On"] = true,
	["on"] = true,
	["Toggles the plugin on and off."] = true,
	["Void Blast Reflected!"] = true,
	["Void Sentinel"] = true,
	["Void Blast"] = true,
} end )

--虚空戒卫
--spellId: 46161
L:RegisterTranslations("zhCN", function() return {
	["Void Blast Reflect"] = "虚空冲击反射",
	["voidblastreflect"] = "虚空冲击反射",
	["Optionally broadcast the raid when void blast is reflected."] = "选择当虚空冲击被反射时在团队中发出通知警报",
	["broadcast"] = "广播",
	["Broadcast"] = "广播",
	["Toggle broadcasting."] = "切换广播",
	["On"] = "打开",
	["on"] = "打开",
	["Toggles the plugin on and off."] = "切换插件打开/关闭",
	["Void Blast Reflected!"] = "虚空冲击反射!",
	["Void Sentinel"] = "虚空戒卫",
	["Void Blast"] = "虚空冲击",
} end )

L:RegisterTranslations("koKR", function() return {
	["Void Blast Reflect"] = "공허의 폭발 반사",
	["voidblastreflect"] = "공허 폭발 반사",
	["Optionally broadcast the raid when void blast is reflected."] = "공허의 폭발을 반사하면 공격대에 알립니다.",
	["broadcast"] = "알림",
	["Broadcast"] = "알림",
	["Toggle broadcasting."] = "알림 전환",
	["On"] = "사용",
	["on"] = "사용",
	["Toggles the plugin on and off."] = "플러그인을 켜거나 끕니다.",
	["Void Blast Reflected!"] = "공허의 폭발 반사!",
	["Void Sentinel"] = "공허의 파수병",
	["Void Blast"] = "공허의 폭발",
} end )

BigWigsVoidBlastReflect = BigWigs:NewModule(name)
BigWigsVoidBlastReflect.defaultDB = {
	broadcast = false,
	on = true,
}

BigWigsVoidBlastReflect.consoleCmd = L["voidblastreflect"]
BigWigsVoidBlastReflect.consoleOptions = {
	type = "group",
	name = L["Void Blast Reflect"],
	desc = L["Optionally broadcast the raid when void blast is reflected."],
	args   = {
		[L["on"]] = {
			type = "toggle",
			name = L["On"],
			desc = L["Toggles the plugin on and off."],
			get = function() return BigWigsVoidBlastReflect:IsEventRegistered("COMBAT_LOG_EVENT_UNFILTERED") end,
			set = function(v)
				BigWigsVoidBlastReflect.db.profile.on = v
				if v then
					BigWigsVoidBlastReflect:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
				else
					BigWigsVoidBlastReflect:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
				end
			end,
		},
		[L["broadcast"]] = {
			type = "toggle",
			name = L["Broadcast"],
			desc = L["Toggle broadcasting."],
			get = function() return BigWigsVoidBlastReflect.db.profile.broadcast end,
			set = function(v) BigWigsVoidBlastReflect.db.profile.broadcast = v end,
		},
	}
}
BigWigsVoidBlastReflect.revision = 1
BigWigsVoidBlastReflect.external = true

function BigWigsVoidBlastReflect:OnEnable()
	if self.db.profile.on then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	end
	times = {}
end

--6/1 23:14:11.296  SPELL_DAMAGE,0xF1300064AC001A51,"Void Sentinel",0x10a48,0xF1300064AC001A51,"Void Sentinel",0x10a48,46161,"Void Blast",0x20,13490,32,0,0,0,nil,nil,nil

function BigWigsVoidBlastReflect:COMBAT_LOG_EVENT_UNFILTERED(tstamp,event,srcGUID,srcName,srcFlags,destGUID,destName,destFlags,...)
	if not UnitInRaid("player") then return end
	
	if srcName and srcName == L["Void Sentinel"] and destName and destName == L["Void Sentinel"] then 
		if event == "SPELL_DAMAGE" then
			local spell = select(2,...);
			if spell and spell == L["Void Blast"] then
				local spellid = select(1,...);
				local spellicon = nil;
				if spellid and type(spellid) == "number" then
					spellicon = select(3, GetSpellInfo(spellid));
				end
				
				self:TriggerEvent("BigWigs_Message", L["Void Blast Reflected!"], "Orange", not self.db.profile.broadcast)

				return;
			end
		end
	end
end