--[[
  Guild Event Manager (GEM) - Version 3
  by Kiki from European "Conseil des Ombres" (Horde)
  ----------------------------------------
  Sorting plugin - Uses subscription TimeStamp as priority, without minimums check
]]

local GEM3_SORT_NAME = nil;
local GEM3_SORT_HELP = nil;
if ( GetLocale() == "frFR" ) then
GEM3_SORT_NAME = "Date";
GEM3_SORT_HELP = "Tri les joueurs en fonction de leur date d'inscription uniquement.";
--[[elseif ( GetLocale() == "deDE" ) then
GEM3_SORT_NAME = "Datum";
GEM3_SORT_HELP = "Sortiert Spieler unter Beachtung ihrer Anmeldezeit.";]]
else
GEM3_SORT_NAME = "Date only";
GEM3_SORT_HELP = "Sort players using their subscription time (no minimum check).";
end

-- Quick acces
local tinsert = table.insert;
local tremove = table.remove;
local tsort = table.sort;

--------------- Internal functions ---------------

local function _GEM3_SORT_CheckGlobalLimit(event,player,count)
  local clas_lim = event.classes[player.class];
  local rol_lim = event.roles[player.role];

  -- Check if exceeding class limit
  if(count > event.max_count) -- Global limit reached
  then
    if((clas_lim.min and clas_lim.sort_stamp_count <= clas_lim.min)
       or (rol_lim and rol_lim.min and rol_lim.sort_stamp_count <= rol_lim.min))
    then
      GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Global limit reached but minima fails if I remove this player, keeping him as titular");
      return true;
    end
    GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Global limit reached for "..player.name.." ("..count.." > "..event.max_count..")");
    return false;
  end
  return true; -- Check is OK
end

local function _GEM3_SORT_SortList(tab1,tab2)
  if(tab1.lead_force_queue == GEM3_SUB_FORCE_QUEUE_TITULAR) -- 1 forced titular
  then
    if(tab2.lead_force_queue ~= GEM3_SUB_FORCE_QUEUE_TITULAR) -- but 2 not forced
    then
      return true; -- Sort 1 before
    end
  elseif(tab2.lead_force_queue == GEM3_SUB_FORCE_QUEUE_TITULAR) -- 2 forced titular (but not 1, otherwise it would have been trapped before)
  then
    return false; -- Sort 2 before
  end
  return tab1.main_time < tab2.main_time;
end

local function _GEM3_SORT_SetTitular(event,tits,tab)
  local clas_lim = event.classes[tab.class];
  local rol_lim = event.roles[tab.role];

  clas_lim.sort_stamp_count = clas_lim.sort_stamp_count + 1; -- Increase class limit
  if(rol_lim)
  then
    rol_lim.sort_stamp_count = rol_lim.sort_stamp_count + 1; -- Increase role limit
  end
  tinsert(tits,tab);
end

local function _GEM3_SORT_SetSubstitute(event,subs,tab)
  tinsert(subs,tab);
end

local function _GEM3_SORT_MoveTitularToSubstitute(event,tits,subs,tab,pos)
  local clas_lim = event.classes[tab.class];
  local rol_lim = event.roles[tab.role];

  clas_lim.sort_stamp_count = clas_lim.sort_stamp_count - 1; -- Decrease class limit
  if(rol_lim)
  then
    rol_lim.sort_stamp_count = rol_lim.sort_stamp_count - 1; -- Decrease role limit
  end

  tinsert(subs,tab);
  tremove(tits,pos);
end

local function _GEM3_SORT_Sort(event)
  local tits = GA_GetTable();
  local subs = GA_GetTable();
  local repl = GA_GetTable();
  local rejs = GA_GetTable();

  GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Entering Sorting function");
  
  -- Reset some internal data
  for n,lim in pairs(event.classes)
  do
    lim.sort_stamp_count = 0;
  end
  for n,lim in pairs(event.roles)
  do
    lim.sort_stamp_count = 0;
  end

  -- Create list of all players eligible for Titular and Substitutes queues
  for name,tab in pairs(event.players)
  do
    GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Processing player "..name..": LeadForced="..tostring(tab.lead_force_queue).." Forced="..tostring(tab.force_queue));
    if(tab.lead_force_queue == GEM3_SUB_FORCE_QUEUE_TITULAR and tab.force_queue ~= GEM3_SUB_FORCE_QUEUE_NOT_COMING) -- Forced Tit by leader, and still coming
    then
      _GEM3_SORT_SetTitular(event,tits,tab);
      GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Forced titular by leader");
    elseif(tab.lead_force_queue == GEM3_SUB_FORCE_QUEUE_SUBSTITUTE and tab.force_queue ~= GEM3_SUB_FORCE_QUEUE_NOT_COMING) -- Forced Sub by leader, and still coming
    then
      _GEM3_SORT_SetSubstitute(event,subs,tab);
      GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Forced substiture by leader");
    elseif(tab.lead_force_queue == GEM3_SUB_FORCE_QUEUE_REPLACEMENT and tab.force_queue ~= GEM3_SUB_FORCE_QUEUE_NOT_COMING) -- Forced Repl by leader, and still coming
    then
      tinsert(repl,tab);
      GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Forced replacement by leader");
    elseif(tab.lead_force_queue == GEM3_SUB_FORCE_QUEUE_NOT_COMING) -- Forced NC by leader
    then
      tinsert(rejs,tab);
      GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Forced reject by leader");
    elseif(tab.lead_force_queue == nil)
    then
      GEM3_ChatWarning("Plugin Stamp: tab.lead_force_queue is nil for "..name);
    else -- Use player's queue (not forced by leader, or player choose NOT_COMING)
      if(tab.force_queue == GEM3_SUB_FORCE_QUEUE_TITULAR) -- Player wants to be titular
      then
        GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Player wants any available queue");
        _GEM3_SORT_SetTitular(event,tits,tab);
      elseif(tab.force_queue == GEM3_SUB_FORCE_QUEUE_SUBSTITUTE) -- Player want to be forced into Sub
      then
        _GEM3_SORT_SetSubstitute(event,subs,tab);
        GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Forced substitute by player");
      elseif(tab.force_queue == GEM3_SUB_FORCE_QUEUE_REPLACEMENT) -- Player want to be forced into Repl
      then
        tinsert(repl,tab);
        GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Forced replacement by player");
      elseif(tab.force_queue == GEM3_SUB_FORCE_QUEUE_NOT_COMING) -- Player don't want to come
      then
        tinsert(rejs,tab);
        GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Forced reject by player");
      end
    end
  end

  -- Sort tits by stamp
  tsort(tits,_GEM3_SORT_SortList);
  
  local i = #tits; -- Start from last
  
  -- Move limit exceeding Tits intos Subs
  while(i ~= 0)
  do
    local tab = tits[i];
    GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Check limits for player "..tab.name.." (i="..i..", count="..#tits..")");
    if(not _GEM3_SORT_CheckGlobalLimit(event,tab,#tits)) -- Global limit reached
    then
      _GEM3_SORT_MoveTitularToSubstitute(event,tits,subs,tab,i);
    end
    i = i - 1; -- Next!
  end
  
  -- Sort other queues by stamp
  tsort(subs,_GEM3_SORT_SortList);
  tsort(repl,_GEM3_SORT_SortList);
  tsort(rejs,_GEM3_SORT_SortList);
  
  -- Set queue pos for all queues
  for i,tab in ipairs(tits)
  do
    tab.current_queue = GEM3_SUB_STATE_TITULAR;
    tab.queue_pos = i;
    GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Titular #"..i.." is "..tab.name);
  end
  for i,tab in ipairs(subs)
  do
    tab.current_queue = GEM3_SUB_STATE_SUBSTITUTE;
    tab.queue_pos = i;
    GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Substitute #"..i.." is "..tab.name);
  end
  for i,tab in ipairs(repl)
  do
    tab.current_queue = GEM3_SUB_STATE_REPLACEMENT;
    tab.queue_pos = i;
    GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Replacement #"..i.." is "..tab.name);
  end
  for i,tab in ipairs(rejs)
  do
    tab.current_queue = GEM3_SUB_STATE_NOT_COMING;
    tab.queue_pos = i;
    GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: NotComing #"..i.." is "..tab.name);
  end

  -- Release tables
  GA_ReleaseTable(tits);
  GA_ReleaseTable(subs);
  GA_ReleaseTable(repl);
  GA_ReleaseTable(rejs);

  GEM3_ChatDebug(GEM3_DEBUG_SORT,"Plugin Stamp: Leaving Sorting function");
end


--------------- Plugin structure ---------------

local GEM3_SORT_Stamp = {
  --[[
    Name parameter. [MANDATORY]
     Displayed name of the sorting plugin.
  ]]
  Name = GEM3_SORT_NAME;
  
  --[[
    SortType parameter. [MANDATORY]
     Internal code for the sorting type (must be unique).
     Please request a unique SortType value to Kiki, if you plan on creating a sorting plugin yourself.
  ]]
  SortType = 2;
  
  --[[
    Subscribers sorting function. [MANDATORY]
     Sorts all passed players in two lists.
     Params :
      - event : Event (STRUCT)
  ]]
  Sort = function(event)
    _GEM3_SORT_Sort(event);
  end;
  
  --[[
    Configure function. [OPTIONAL]
     Configures the plugin.
  ]]
  --Configure = function()
  --end;

  --[[
    Help parameter. [OPTIONAL]
     Help string displayed when you mouse over the sort type.
  ]]
  Help = GEM3_SORT_HELP;
};

-- Register Plugin
GEM3_SORT_RegisterPlugin(GEM3_SORT_Stamp);

